// 8-bell MBI using internal pullups.
// Sends ascii characters to PC (or XBee or whatever) via pins 0 and 1.
// Single LED (pin 13) that lights momentarily when any signal is received.

#include "PinChangeInt.h"

#define BELLS 8

#define BELL1 2  // treble
#define BELL2 3  // 2nd
#define BELL3 4  // 3rd
#define BELL4 5  // 4th
#define BELL5 6  // 5th
#define BELL6 7  // 6th
#define BELL7 8  // 7th
#define BELL8 9  // tenor 

#define LED 13

// Reed switches bounce. 
// A debounce filter will therefore cut a lot of unnecessary traffic on the serial port.
#define DEBOUNCE_MS 2
#define LED_DURATION 100

unsigned long time_of_signal;
unsigned long time_of_last_signal[BELLS];
char char_to_send[BELLS] = {'1','2','3','4','5','6','7','8'};
int bells[BELLS] = {BELL1,BELL2,BELL3,BELL4,BELL5,BELL6,BELL7,BELL8};

void setup()
{
  pinMode(BELL1,INPUT_PULLUP);
  pinMode(BELL2,INPUT_PULLUP);
  pinMode(BELL3,INPUT_PULLUP);
  pinMode(BELL4,INPUT_PULLUP);
  pinMode(BELL5,INPUT_PULLUP);
  pinMode(BELL6,INPUT_PULLUP);
  pinMode(BELL7,INPUT_PULLUP);
  pinMode(BELL8,INPUT_PULLUP);

  pinMode(LED,OUTPUT);

  for (int i=0;i<BELLS;++i)
    time_of_last_signal[i] = 0;

  time_of_signal = 0;

  digitalWrite(LED,LOW);

  attachInterrupt(digitalPinToInterrupt(BELL1), event1, CHANGE);
  attachInterrupt(digitalPinToInterrupt(BELL2), event2, CHANGE);
  attachPinChangeInterrupt(BELL3,event3,CHANGE);
  attachPinChangeInterrupt(BELL4,event4,CHANGE);
  attachPinChangeInterrupt(BELL5,event5,CHANGE);
  attachPinChangeInterrupt(BELL6,event6,CHANGE);
  attachPinChangeInterrupt(BELL7,event7,CHANGE);
  attachPinChangeInterrupt(BELL8,event8,CHANGE);

  Serial.begin(2400);
}

void loop()
{
  digitalWrite(LED,(millis() - time_of_signal < LED_DURATION) ? HIGH : LOW);
}

void event1() {handle_event(0);}
void event2() {handle_event(1);}
void event3() {handle_event(2);}
void event4() {handle_event(3);}
void event5() {handle_event(4);}
void event6() {handle_event(5);}
void event7() {handle_event(6);}
void event8() {handle_event(7);}

void handle_event(int bell)
{
  time_of_signal = millis();
  if (time_of_signal - time_of_last_signal[bell] > DEBOUNCE_MS) {
    Serial.write(char_to_send[bell]);
    time_of_last_signal[bell] = time_of_signal;
  }
}

